package de.w665.biblenotes.rest.security;

import de.w665.biblenotes.service.AuthenticationService;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.security.web.util.matcher.RequestMatcher;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;
import java.util.Collections;
import java.util.List;

@Component
@Slf4j
@RequiredArgsConstructor
public class JwtAuthenticationFilter extends OncePerRequestFilter {

    private final AuthenticationService authenticationService;
    private final RequestMatcher requestMatcher = new AntPathRequestMatcher("/api/v1/secure/**");

    @Override
    protected void doFilterInternal(@NotNull HttpServletRequest request, @NotNull HttpServletResponse response, @NotNull FilterChain filterChain) throws ServletException, IOException {

        logger.debug("Filtering request: " + request.getRequestURI());

        // Skip filter for all paths except the secure path
        if(!requestMatcher.matches(request)) {
            logger.debug("Request does not match the secure path. Skipping JWT authentication.");
            filterChain.doFilter(request, response);
            return;
        }

        // Extract the JWT token from the request
        String jwt = getJwtFromRequest(request);

        if (jwt == null) { // Check if the JWT token is missing
            response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
            response.setContentType("application/json");
            response.getWriter().write("{\"error\": \"Unauthorized\", \"message\": \"JWT token is missing.\"}");
            logger.warn("Unauthorized: JWT token is missing.");
            return;
        }

        if (!authenticationService.validateToken(jwt)) { // Validate the JWT token
            response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
            response.setContentType("application/json");
            response.getWriter().write("{\"error\": \"Unauthorized\", \"message\": \"JWT token is invalid.\"}");
            logger.warn("Unauthorized: JWT token is invalid.");
            return;
        }

        String username = authenticationService.extractSubject(jwt);
        // Extract the role from the JWT and set it to Spring AuthenticationContext for access control
        String role = authenticationService.getClaimValue(jwt, "role", String.class);
        List<GrantedAuthority> authorities = Collections.singletonList(new SimpleGrantedAuthority("ROLE_" + role));

        JwtAuthenticationToken auth = new JwtAuthenticationToken(username, jwt, authorities);
        auth.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));
        SecurityContextHolder.getContext().setAuthentication(auth);

        // SUCCESSFUL AUTHENTICATION
        filterChain.doFilter(request, response);
    }

    private String getJwtFromRequest(HttpServletRequest request) {
        String bearerToken = request.getHeader("Authorization");
        if (bearerToken != null && bearerToken.startsWith("Bearer ")) {
            return bearerToken.substring(7);
        }
        return null;
    }
}
